from typing import List, Optional

from sqlalchemy.orm import selectinload
from sqlalchemy.orm.query import Query
from sqlalchemy.sql.elements import BinaryExpression

from rfhub2.db.base import Collection
from rfhub2.db.repository.base_repository import BaseRepository
from rfhub2.db.repository.query_utils import glob_to_sql


class CollectionRepository(BaseRepository):
    @property
    def _items(self) -> Query:
        return self.session.query(Collection).options(selectinload(Collection.keywords))

    def _id_filter(self, item_id: int) -> BinaryExpression:
        return Collection.id == item_id

    def get_all(
        self,
        *,
        pattern: Optional[str] = None,
        libtype: Optional[str] = None,
        skip: int = 0,
        limit: int = 100,
    ) -> List[Collection]:
        filter_criteria = []
        if pattern:
            filter_criteria.append(Collection.name.ilike(glob_to_sql(pattern)))
        if libtype:
            filter_criteria.append(Collection.type.ilike(glob_to_sql(libtype)))
        return (
            self._items.filter(*filter_criteria)
            .order_by(Collection.name)
            .offset(skip)
            .limit(limit)
            .all()
        )



import click
from pathlib import Path
from typing import Tuple

from rfhub2.cli.api_client import Client
from rfhub2.cli.rfhub_importer import RfhubImporter


@click.command()
@click.option(
    "-a",
    "--app-url",
    type=click.STRING,
    default="http://localhost:8000",
    help="Specifies IP, URI or host of rfhub2 web application. Default value is http://localhost:8000.",
)
@click.option(
    "-u",
    "--user",
    type=click.STRING,
    default="rfhub",
    help="Specifies rfhub2 user to authenticate on endpoints that requires that. Default value is rfhub.",
)
@click.option(
    "-p",
    "--password",
    type=click.STRING,
    default="rfhub",
    help="Specifies rfhub2 password to authenticate on endpoints that requires that. Default value is rfhub.",
)
@click.option(
    "--no-installed-keywords",
    type=click.BOOL,
    default=False,
    is_flag=True,
    help="Flag specifying if package should skip loading commonly installed libraries, "
    "such as such as BuiltIn, Collections, DateTime etc.",
)
@click.option(
    "--no-db-flush",
    type=click.BOOL,
    default=False,
    is_flag=True,
    help="Flag specifying if package should delete from rfhub2 all existing libraries.",
)
@click.argument("paths", nargs=-1, type=click.Path(exists=True))
def main(
    app_url: str,
    user: str,
    password: str,
    paths: Tuple[Path, ...],
    no_db_flush: bool,
    no_installed_keywords: bool,
) -> None:
    """Package to populate rfhub2 with robot framework keywords
       from libraries and resource files."""
    client = Client(app_url, user, password)
    rfhub_importer = RfhubImporter(client, paths, no_installed_keywords)
    if not no_db_flush:
        rfhub_importer.delete_all_collections()
    loaded_collections, loaded_keywords = rfhub_importer.import_libraries()
    print(
        f"\nSuccessfully loaded {loaded_collections} collections with {loaded_keywords} keywords."
    )



from fastapi import FastAPI
from starlette.staticfiles import StaticFiles
from starlette.middleware.cors import CORSMiddleware

from rfhub2 import config
from rfhub2.api.router import api_router
from rfhub2.api.middleware.db_session_middleware import DbSessionMiddleware
from rfhub2.ui.ui_router import router as ui_router
from rfhub2.utils import abs_path
from rfhub2.version import version


def create_app() -> FastAPI:
    app = FastAPI(title=config.APP_TITLE, version=version)
    app.mount("/static", StaticFiles(directory=abs_path("static")), name="static")
    app.include_router(ui_router)
    app.include_router(api_router, prefix="/api/v1")
    app.add_middleware(CORSMiddleware, allow_origins=["*"])
    app.add_middleware(DbSessionMiddleware)
    return app



from fastapi import APIRouter, Depends
from starlette.responses import Response

from rfhub2.api.utils.db import db_healthcheck
from rfhub2.model import Healthcheck

router = APIRouter()


@router.get("/", response_model=Healthcheck)
def healthcheck(response: Response, db_status: bool = Depends(db_healthcheck)):
    if db_status:
        return Healthcheck(**{"db": "ok"})
    else:
        response.status_code = 503
        return Healthcheck(**{"db": "failure"})



from pathlib import Path
import re
from robot.errors import DataError
from robot.libdocpkg import LibraryDocumentation
import robot.libraries
from typing import Dict, List, Set, Tuple

from .api_client import Client


RESOURCE_PATTERNS = {".robot", ".txt", ".tsv", ".resource"}
ALL_PATTERNS = RESOURCE_PATTERNS | {".xml", ".py"}
EXCLUDED_LIBRARIES = {
    "remote.py",
    "reserved.py",
    "dialogs.py",
    "dialogs_jy.py",
    "dialogs_py.py",
    "dialogs_ipy.py",
    "setup.py",
}
INIT_FILES = {"__init__.txt", "__init__.robot", "__init__.html", "__init__.tsv"}


class RfhubImporter(object):
    def __init__(
        self, client: Client, paths: Tuple[Path, ...], no_installed_keywords: bool
    ) -> None:
        self.paths = paths
        self.no_installed_keywords = no_installed_keywords
        self.client = client

    def delete_all_collections(self) -> Set[int]:
        """
        Deletes all existing collections.
        """
        collections_id = set()
        while len(self.client.get_collections()) > 0:
            collections_id.update(self._delete_collections())
        return collections_id

    def _delete_collections(self) -> Set[int]:
        """
        Helper method to delete all existing callections.
        """
        collections = self.client.get_collections()
        collections_id = {collection["id"] for collection in collections}
        for id in collections_id:
            self.client.delete_collection(id)
        return collections_id

    def import_libraries(self) -> Tuple[int, int]:
        """
        Import libraries to application from paths specified when invoking client.
        :return: Number of libraries loaded
        """
        libraries_paths = self.get_libraries_paths()
        collections = self.create_collections(libraries_paths)
        loaded_collections = self.add_collections(collections)
        return len(loaded_collections), sum(d["keywords"] for d in loaded_collections)

    def get_libraries_paths(self) -> Set[Path]:
        """
        Traverses all given paths and returns set with paths
        pointing to libraries to import to app.
        :return: Set of Paths object pointing to libraries to import
        """
        libraries_paths = set()
        for path in self.paths:
            libraries_paths.update(self._traverse_paths(Path(path)))
        if not self.no_installed_keywords:
            libdir = Path(robot.libraries.__file__).parent
            libraries_paths.update(self._traverse_paths(Path(libdir)))
        return libraries_paths

    def _traverse_paths(self, path: Path) -> Set[Path]:
        """
        Traverses through paths and adds libraries to rfhub.
        Helper function for get_library_paths.
        """
        valid_lib_paths = set()
        if self._is_library_with_init(path):
            valid_lib_paths.add(path)
        else:
            for item in path.iterdir():
                if item.is_dir():
                    if self._is_library_with_init(item):
                        valid_lib_paths.add(item)
                    else:
                        valid_lib_paths.update(self._traverse_paths(item))
                elif (
                    item.is_file()
                    and self._is_robot_keyword_file(item)
                    and not self._should_ignore(item)
                ):
                    valid_lib_paths.add(item)
        return valid_lib_paths

    def create_collections(self, paths: Set[Path]) -> List[Dict]:
        """
        Creates list of Collection objects from set of provided paths.
        :param paths: set of paths
        :return: list of Collection objects
        """
        collections = []
        for path in paths:
            try:
                collection = self.create_collection(path)
                collections.append(collection)
            except (DataError, SystemExit) as ex:
                print(
                    f"Failed to create collection from path {path}\n"
                    f"{type(ex).__name__}, {ex.args}"
                )
        return collections

    def create_collection(self, path: Path) -> Dict:
        """
        Creates Collection object from provided path.
        :param path: Path
        :return: Collection object
        """
        libdoc = LibraryDocumentation(str(path))
        serialised_keywords = self._serialise_keywords(libdoc)
        return self._serialise_libdoc(libdoc, str(path), serialised_keywords)

    def add_collections(self, collections: Dict) -> List[Dict[str, int]]:
        """
        Adds collections and keywords from provided list to app.
        :param collections: List of collections object
        :return: list of dictionaries with collection name and number of keywords.
        """
        loaded_collections = []
        for collection in collections:
            coll_req = self.client.add_collection(collection)
            if coll_req[0] != 201:
                print(coll_req[1]["detail"])
                raise StopIteration
            collection_id = coll_req[1]["id"]
            for keyword in collection["keywords"]:
                keyword["collection_id"] = collection_id
                self.client.add_keyword(keyword)
            loaded_collections.append(
                {"name": collection["name"], "keywords": len(collection["keywords"])}
            )
            print(
                f'{collection["name"]} library with {len(collection["keywords"])} keywords loaded.'
            )
        return loaded_collections

    def _serialise_libdoc(self, libdoc: Dict, path: str, keywords: Dict) -> Dict:
        """
        Serialises libdoc object to Collection object.
        :param libdoc: libdoc input object
        :param path: library path
        :return: Collection object
        """

        lib_dict = libdoc.__dict__
        lib_dict["doc_format"] = lib_dict.pop("_setter__doc_format")
        for key in ("_setter__keywords", "inits", "named_args"):
            lib_dict.pop(key)
        lib_dict["path"] = path
        lib_dict["keywords"] = keywords
        return lib_dict

    def _serialise_keywords(self, libdoc: Dict) -> Dict:
        """
        Serialises keywords to Keyword object.
        :param :libdoc input object
        :return: Keyword object
        """

        keywords = [keyword.__dict__ for keyword in libdoc.keywords]
        for keyword in keywords:
            keyword.pop("tags")
            if keyword["args"]:
                keyword["args"] = str(
                    [
                        str(item).replace("'", "").replace('"', "")
                        for item in keyword["args"]
                    ]
                ).replace("'", '"')
            else:
                keyword["args"] = ""
        return keywords

    @staticmethod
    def _is_library_with_init(path: Path) -> bool:
        return (path / "__init__.py").is_file() and len(
            LibraryDocumentation(str(path)).keywords
        ) > 0

    def _is_robot_keyword_file(self, file: Path) -> bool:
        return (
            self._is_library_file(file)
            or self._is_libdoc_file(file)
            or self._is_resource_file(file)
        )

    @staticmethod
    def _is_library_file(file: Path) -> bool:
        return file.suffix == ".py" and file.name != "__init__.py"

    @staticmethod
    def _is_libdoc_file(file: Path) -> bool:
        """Return true if an xml file looks like a libdoc file"""
        # inefficient since we end up reading the file twice,
        # but it's fast enough for our purposes, and prevents
        # us from doing a full parse of files that are obviously
        # not libdoc files
        if file.suffix == ".xml":
            with open(file, "r", encoding="utf-8", errors="ignore") as f:
                # read the first few lines; if we don't see
                # what looks like libdoc data, return false
                data = f.read(200)
                index = data.lower().find("<keywordspec ")
                return index > 0
        return False

    @staticmethod
    def _should_ignore(file: Path) -> bool:
        """Return True if a given library name should be ignored
        This is necessary because not all files we find in the library
        folder are libraries.
        """
        filename = file.name.lower()
        return (
            filename.startswith("deprecated")
            or filename.startswith("_")
            or filename in EXCLUDED_LIBRARIES
        )

    @staticmethod
    def _is_resource_file(file: Path) -> bool:
        """Returns true if the file has a keyword table but not a testcase table."""
        # inefficient since we end up reading the file twice,
        # but it's fast enough for our purposes, and prevents
        # us from doing a full parse of files that are obviously
        # not robot files

        if file.name not in INIT_FILES and file.suffix in RESOURCE_PATTERNS:
            with open(file, "r") as f:
                data = f.read()
                return not RfhubImporter._has_test_case_table(
                    data
                ) and RfhubImporter._has_keyword_table(data)
        return False

    @staticmethod
    def _has_keyword_table(data: str) -> bool:
        """Returns true if file has keyword or user keyword table"""
        return (
            re.search(
                r"^\*+\s*((?:User )?Keywords?)", data, re.MULTILINE | re.IGNORECASE
            )
            is not None
        )

    @staticmethod
    def _has_test_case_table(data: str) -> bool:
        """Returns true if file has keyword or user keyword table"""
        return (
            re.search(r"^\*+\s*(Test Cases?)", data, re.MULTILINE | re.IGNORECASE)
            is not None
        )



from pydantic import BaseModel
from typing import List, Optional


class VersionInfo(BaseModel):
    title: str
    version: str


class Healthcheck(BaseModel):
    db: str


class NestedCollection(BaseModel):
    id: int
    name: str

    class Config:
        orm_mode = True


class KeywordUpdate(BaseModel):
    name: str
    doc: Optional[str]
    args: Optional[str]

    class Config:
        orm_mode = True


class KeywordCreate(KeywordUpdate):
    collection_id: int


class CollectionUpdate(BaseModel):
    name: str
    type: Optional[str]
    version: Optional[str]
    scope: Optional[str]
    named_args: Optional[str]
    path: Optional[str]
    doc: Optional[str]
    doc_format: Optional[str]


class NestedKeyword(KeywordUpdate):
    id: int
    synopsis: Optional[str]
    html_doc: Optional[str]
    arg_string: Optional[str]


class Collection(NestedCollection, CollectionUpdate):
    keywords: List[NestedKeyword]
    synopsis: Optional[str]
    html_doc: Optional[str]


class Keyword(NestedKeyword):
    collection: NestedCollection



from sqlalchemy.orm.session import Session

from rfhub2.db.base import Collection, Keyword


def recreate_data(session: Session) -> None:
    session.query(Keyword).delete()
    session.query(Collection).delete()
    keywords = [
        Keyword(
            name="Test setup",
            doc="Prepare test environment, use teardown after this one",
        ),
        Keyword(name="Some keyword", doc="Perform some check"),
        Keyword(name="Teardown", doc="Clean up environment"),
    ]
    collections = [
        Collection(name="First collection", type="robot", keywords=keywords),
        Collection(name="Second collection", type="Robot"),
        Collection(name="Third", type="Library"),
    ]
    session.add_all(collections)
    session.commit()



from sqlalchemy import create_engine, event
from sqlalchemy.engine import Engine
from sqlalchemy.orm import scoped_session, sessionmaker
import sqlite3

from rfhub2 import config


def create_sqlalchemy_engine(db_uri: str) -> Engine:
    if db_uri.startswith("sqlite://"):
        engine_kwargs = {"connect_args": {"check_same_thread": False}}
    else:
        engine_kwargs = {}
    return create_engine(
        config.SQLALCHEMY_DB_URI, pool_pre_ping=True, echo=False, **engine_kwargs
    )


@event.listens_for(Engine, "connect")
def set_sqlite_fk_pragma(db_api_connection, _):
    """
    Setting foreign keys pragma is required to enable on delete cascade behavior
    for foreign key fields which is by default disabled
    """
    if isinstance(db_api_connection, sqlite3.Connection):
        cursor = db_api_connection.cursor()
        cursor.execute("PRAGMA foreign_keys=ON;")
        cursor.close()


engine = create_sqlalchemy_engine(config.SQLALCHEMY_DB_URI)

db_session = scoped_session(
    sessionmaker(autocommit=False, autoflush=False, bind=engine)
)
Session = sessionmaker(autocommit=False, autoflush=False, bind=engine)



from sqlalchemy import Column, Integer, Sequence, Text
from sqlalchemy.orm import relationship

from rfhub2.db.model.base_class import Base
from rfhub2.db.model.doc_mixin import DocMixin


class Collection(Base, DocMixin):
    id = Column(Integer, Sequence("collection_id_seq"), primary_key=True)
    name = Column(Text, index=True)
    type = Column(Text)
    version = Column(Text)
    scope = Column(Text)
    named_args = Column(Text)
    path = Column(Text)
    doc = Column(Text)
    doc_format = Column(Text)
    keywords = relationship(
        "Keyword",
        backref="collection",
        cascade="all, delete-orphan",
        passive_deletes=True,
        order_by="Keyword.name",
    )

    def __str__(self):  # pragma: no cover
        return (
            f"Collection({self.id},{self.name},{self.type},{self.version},"
            + f"{self.scope},{self.named_args},{self.path},{self.doc},{self.doc_format})"
        )

    def __repr__(self):  # pragma: no cover
        return str(self)



import json
from sqlalchemy import Column, ForeignKey, Integer, Sequence, Text

from rfhub2.db.model.base_class import Base
from rfhub2.db.model.doc_mixin import DocMixin


class Keyword(Base, DocMixin):
    id = Column(Integer, Sequence("keyword_id_seq"), primary_key=True)
    name = Column(Text, index=True)
    doc = Column(Text)
    args = Column(Text)
    collection_id = Column(
        Integer, ForeignKey("collection.id", ondelete="CASCADE"), nullable=False
    )

    def __str__(self):  # pragma: no cover
        return f"Keyword({self.id},{self.name},{self.doc},{self.args},{self.collection_id})"

    def __repr__(self):  # pragma: no cover
        return str(self)

    @property
    def arg_string(self) -> str:
        """
        Old implementation saves args list as JSON in text field, this is more readable representation for UI
        """
        return ", ".join(json.loads(self.args)) if self.args else ""



from robot.libdocpkg.htmlwriter import DocToHtml


class DocMixin:
    @property
    def synopsis(self) -> str:
        return self.doc.splitlines()[0] if self.doc else ""

    @property
    def html_doc(self) -> str:
        return DocToHtml("ROBOT")(self.doc) if self.doc else ""



import os

APP_TITLE = os.getenv("RFHUB_APP_TITLE", "rfhub2")
APP_INTERFACE = os.getenv("RFHUB_APP_INTERFACE", "0.0.0.0")
APP_PORT = int(os.getenv("PORT", 8000))
APP_LOG_LEVEL = os.getenv("RFHUB_APP_LOG_LEVEL", "info")
BASIC_AUTH_USER = os.getenv("RFHUB_BASIC_AUTH_USER", "rfhub")
BASIC_AUTH_PASSWORD = os.getenv("RFHUB_BASIC_AUTH_PASSWORD", "rfhub")
SQLALCHEMY_DB_URI = os.getenv("RFHUB_DB_URI", "sqlite:///test.db")



from setuptools import setup

filename = 'rfhub2/version.py'
exec(open(filename).read())

setup(
    name='rfhub2',
    version=version,
    author='Pawel Bylicki, Maciej Wiczk',
    author_email='pawelkbylicki@gmail.com, maciejwiczk@gmail.com',
    url='https://github.com/pbylicki/rfhub2/',
    keywords='robotframework',
    license='Apache License 2.0',
    description='Webserver for robot framework and python assets documentation',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    zip_safe=True,
    include_package_data=True,
    install_requires=[
        'aiofiles>=0.4.0',
        'Click>=7.0',
        'fastapi>=0.30.0',
        'robotframework>=3.0.0',
        'SQLAlchemy>=1.1.0',
        'requests>=2.10.0',
        'uvicorn>=0.7.1'
    ],
    extras_require={
        "postgresql": ["psycopg2-binary>=2.7.4"]
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: Unix",
        "Framework :: Robot Framework",
        "Framework :: Robot Framework :: Tool",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Quality Assurance",
        "Intended Audience :: Developers",
    ],
    packages=[
        'rfhub2',
        'rfhub2.api',
        'rfhub2.api.endpoints',
        'rfhub2.api.middleware',
        'rfhub2.api.utils',
        'rfhub2.cli',
        'rfhub2.db',
        'rfhub2.db.model',
        'rfhub2.db.repository',
        'rfhub2.model',
        'rfhub2.ui',
        'rfhub2.utils',
    ],
    scripts=[],
    entry_points={
        'console_scripts': [
            "rfhub2 = rfhub2.__main__:main",
            "rfhub2-cli = rfhub2.cli.__main__:main",
        ]
    }
)



from requests import session, post, get, delete, Response
from typing import Dict, Tuple


API_V1 = "api/v1"
TEST_COLLECTION = {
    "name": "healtcheck_collection",
    "type": "a",
    "version": "a",
    "scope": "a",
    "named_args": "a",
    "path": "a",
    "doc": "a",
    "doc_format": "a",
}


class Client(object):
    """
    API client with methods to populate rfhub2 application.
    """

    def __init__(self, app_url: str, user: str, password: str):
        self.app_url = app_url
        self.session = session()
        self.api_url = f"{self.app_url}/{API_V1}"
        self.session.auth = (user, password)
        self.session.headers = {
            "Content-Type": "application/json",
            "accept": "application/json",
        }

    def get_collections(self) -> Response:
        """
        Gets list of collections object using request get method.
        """
        return self._get_request(endpoint="collections")

    def add_collection(self, data: Dict) -> Dict:
        """
        Adds collection using request post method.
        """
        return self._post_request(endpoint="collections", data=data)

    def delete_collection(self, id: int) -> Response:
        """
        Deletes collection with given id.
        """
        return self._delete_request(endpoint="collections", id=id)

    def add_keyword(self, data: Dict) -> Dict:
        """
        Adds keyword using request post method.
        """
        return self._post_request(endpoint="keywords", data=data)

    def _get_request(self, endpoint: str) -> Dict:
        """
        Sends get request from given endpoint.
        """
        request = self.session.get(url=f"{self.api_url}/{endpoint}/")
        return request.json()

    def _post_request(self, endpoint: str, data: Dict) -> Tuple[int, Dict]:
        """
        Sends post request to collections or keywords endpoint.
        """
        request = self.session.post(url=f"{self.api_url}/{endpoint}/", json=data)
        return request.status_code, request.json()

    def _delete_request(self, endpoint: str, id: int) -> Response:
        """
        Sends delete request to collections or keywords endpoint with item id.
        """
        return self.session.delete(url=f"{self.api_url}/{endpoint}/{id}/")



from fastapi import APIRouter, Depends, HTTPException
from starlette.responses import Response
from typing import List, Optional

from rfhub2.api.utils.auth import is_authenticated
from rfhub2.api.utils.db import get_collection_repository, get_keyword_repository
from rfhub2.api.utils.http import or_404
from rfhub2.db.base import Collection as DBCollection, Keyword as DBKeyword
from rfhub2.db.repository.collection_repository import CollectionRepository
from rfhub2.db.repository.keyword_repository import KeywordRepository
from rfhub2.model import Keyword, KeywordCreate, KeywordUpdate
from rfhub2.ui.search_params import SearchParams

router = APIRouter()


@router.get("/", response_model=List[Keyword])
def get_keywords(
    repository: KeywordRepository = Depends(get_keyword_repository),
    skip: int = 0,
    limit: int = 100,
    pattern: str = None,
    use_doc: bool = True,
):
    keywords: List[DBKeyword] = repository.get_all(
        skip=skip, limit=limit, pattern=pattern, use_doc=use_doc
    )
    return keywords


@router.get("/search/", response_model=List[Keyword])
def search_keywords(
    *,
    repository: KeywordRepository = Depends(get_keyword_repository),
    params: SearchParams = Depends(),
    skip: int = 0,
    limit: int = 100,
):
    return repository.get_all(
        pattern=params.pattern,
        collection_name=params.collection_name,
        use_doc=params.use_doc,
        skip=skip,
        limit=limit,
    )


@router.get("/{id}/", response_model=Keyword)
def get_keyword(
    *, repository: KeywordRepository = Depends(get_keyword_repository), id: int
):
    keyword: Optional[DBKeyword] = repository.get(id)
    return or_404(keyword)


@router.post("/", response_model=Keyword, status_code=201)
def create_keyword(
    *,
    _: bool = Depends(is_authenticated),
    repository: KeywordRepository = Depends(get_keyword_repository),
    collection_repository: CollectionRepository = Depends(get_collection_repository),
    keyword: KeywordCreate,
):
    collection: Optional[DBCollection] = collection_repository.get(
        keyword.collection_id
    )
    if not collection:
        raise HTTPException(status_code=400, detail="Collection does not exist")
    db_keyword: DBKeyword = DBKeyword(**keyword.dict())
    return repository.add(db_keyword)


@router.put("/{id}/", response_model=Keyword)
def update_keyword(
    *,
    _: bool = Depends(is_authenticated),
    repository: KeywordRepository = Depends(get_keyword_repository),
    id: int,
    keyword_update: KeywordUpdate,
):
    db_keyword: DBKeyword = or_404(repository.get(id))
    updated: DBKeyword = repository.update(
        db_keyword, keyword_update.dict(skip_defaults=True)
    )
    return updated


@router.delete("/{id}/")
def delete_keyword(
    *,
    _: bool = Depends(is_authenticated),
    repository: KeywordRepository = Depends(get_keyword_repository),
    id: int,
):
    deleted: int = repository.delete(id)
    if deleted:
        return Response(status_code=204)
    else:
        raise HTTPException(status_code=404)



from fastapi import APIRouter
from starlette.responses import FileResponse

from rfhub2.utils import abs_path

router = APIRouter()


@router.get("/")
@router.get("/search/")
@router.get("/keywords/{collection_id}/")
@router.get("/keywords/{collection_id}/{keyword_id}/")
async def home():
    return FileResponse(abs_path("templates", "index.html"), media_type="text/html")



from fastapi import HTTPException
from typing import Optional, TypeVar

T = TypeVar("T")


def or_404(item: Optional[T]) -> T:
    if not item:
        raise HTTPException(status_code=404)
    return item



from typing import List, Optional

from sqlalchemy import or_
from sqlalchemy.orm import selectinload
from sqlalchemy.orm.query import Query
from sqlalchemy.sql.elements import BinaryExpression

from rfhub2.db.base import Collection, Keyword
from rfhub2.db.repository.base_repository import BaseRepository
from rfhub2.db.repository.query_utils import glob_to_sql


class KeywordRepository(BaseRepository):
    @property
    def _items(self) -> Query:
        return self.session.query(Keyword).options(selectinload(Keyword.collection))

    def _id_filter(self, item_id: int) -> BinaryExpression:
        return Keyword.id == item_id

    def get_all(
        self,
        *,
        pattern: Optional[str] = None,
        collection_name: Optional[str] = None,
        use_doc: bool = True,
        skip: int = 0,
        limit: int = 100,
    ) -> List[Keyword]:
        filter_criteria = []
        if pattern:
            filter_criteria.append(Keyword.name.ilike(glob_to_sql(pattern)))
            if use_doc:
                filter_criteria = [
                    or_(filter_criteria[0], Keyword.doc.ilike(glob_to_sql(pattern)))
                ]
        if collection_name:
            filter_criteria.append(Collection.name.ilike(glob_to_sql(collection_name)))
        return (
            self.session.query(Keyword)
            .join(Keyword.collection)
            .filter(*filter_criteria)
            .order_by(Keyword.name)
            .offset(skip)
            .limit(limit)
            .all()
        )
